;; cl-chmlib - Examples
;; Copyright (C) 2006  Frederic Jolliton <frederic@jolliton.com>
;; 
;; This library is free software; you can redistribute it and/or
;; modify it under the terms of the GNU Lesser General Public
;; License as published by the Free Software Foundation; either
;; version 2.1 of the License, or (at your option) any later version.
;; 
;; This library is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; Lesser General Public License for more details.
;; 
;; You should have received a copy of the GNU Lesser General Public
;; License along with this library; if not, write to the Free Software
;; Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA

(defpackage #:net.tuxee.chmlib-examples
  (:nicknames #:chmlib-examples)
  (:use #:cl #:chmlib)
  (:export #:chm-enum
           #:chm-enumdir))

(in-package #:net.tuxee.chmlib-examples)

;; Example inspired from chmlib-0.37/src/enum{,dir}_chmLib.c

(defvar *flags* `((,+chm-flag-normal+ . "normal")
                  (,+chm-flag-special+ . "special")
                  (,+chm-flag-meta+ . "meta")
                  (,+chm-flag-dirs+ . "dirs")
                  (,+chm-flag-files+ . "files")))

(let ((format "~4<~A~> ~9<~A~> ~9<~A~> ~14A ~A~%"))
  (defun print-directory-header ()
    (format t format "spc" "start" "length" "type" "name")
    (format t format "===" "=====" "======" "====" "===="))
  (defun print-directory-entry (info)
    (let* ((space  (cdr (assoc :space info)))
           (start  (cdr (assoc :start info)))
           (length (cdr (assoc :length info)))
           (type   (cdr (assoc :flags info)))
           (name   (cdr (assoc :path info)))
           (type-name
            ;; Build type name from bits in type
            (loop
               :for separator = "" :then " "
               :with type-name = ""
               :for (flag . flag-name) :in *flags*
               :when (plusp (logand type flag))
               :do (setf type-name (concatenate 'string type-name separator flag-name))
               :finally (return type-name))))
      (format t format space start length type-name name))))

(defun chm-enum (filename)
  (format t "~A:~%" filename)
  (print-directory-header)
  (with-chm (chm filename)
    (chm-enumerate chm +chm-flag-all+
                   (lambda (info) (print-directory-entry info) t))))

(defun chm-enumdir (filename &rest directories)
  (with-chm (chm filename)
    (labels ((print-directory (directory)
               (format t "~A:~%" directory)
               (print-directory-header)
               (chm-enumerate-directory chm directory +chm-flag-all+
                                        (lambda (info) (print-directory-entry info) t))))
      (loop
         :for directory :in (or directories '("/"))
         :do (print-directory directory)))))